package treedisease;

import javax.swing.SwingWorker;
/*
enum TreeState{
    Healthy, 
    Infected,
    Immune,
    Dead
}

class Tree {
    public TreeState state;
    public int daysPassed;
    
    public Tree(TreeState state){
        this.state = state;
    }
}
*/
class Garden {
    final double L; // a kert oldalának hossza (kilométer)
    final int n; // n×n-es rácsban helyezkednek el a fák
    final double p0; // a betegség valószínűsége kezdetben
    final double z; //a napi gyógyulás valószínűsége
    final int x; // ha egy fa x napig nem fertőzött meg másikat, akkor meggyógyul
    final long t_limit;
    final int r; // ekkora sugárban fertőz egy fa, lineáris csökkenéssel (méter)
    private int infected;
    private int dead;
    private boolean heal;
    long t;
    int infectedMax;
    long t_infectedMax;
    byte G[][]; // -2: egészséges, -1: várakozik, 0,1,...,x-1: ennyi napja nem fertőzött meg senkit, x: gyógyulási határt elérte, x+1: gyógyult és immunis
                // -2: egészséges, -1: várakozik, 1...x: beteg és ennyi nap telt el, x+1: elpusztult, x+2 immunis
    
    public Garden(double L, int n, double p0, double z, int x, long t_limit, int r) {
        this.L = L;
        this.n = n;
        this.p0 = p0;
        this.x = x;
        this.t_limit = t_limit;
        this.r = r;
        this.z = z;
        heal = true;
        infected = 0;
        dead = 0;
        t = 0;
        G = new byte[n][n];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                if (Math.random() < p0) {
                    G[i][j] = 0;
                    infected++;
                }
                else G[i][j] = -2;
            }
        }
        infectedMax = infected;
        t_infectedMax = 0;
    }
    
    public void step() {
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                // terjedés
                if (0 <= G[i][j] && G[i][j] <= x) { //ha beteg a fa
                    for (int ii = 0; ii < n; ii++) {    //akkor az összes többi célfát megvizsgáljuk
                        for (int jj = 0; jj < n; jj++) {
                            if (G[ii][jj] == -2) {          // ha a célfa még egészséges
                                double p = 1 - (Math.sqrt((i-ii)*(i-ii) + (j-jj)*(j-jj)) * L) / (n * r/1000.0);
                                if (Math.random() < p) {
                                    G[ii][jj] = -1; infected++; // a célfa megfertőződött
                                }
                            }
                        }
                    }
                }
                
                //telik az idő
                if(0 <= G[i][j] && G[i][j] <= x){ //ha beteg
                    //gyógyulás
                    if(Math.random() < z){          //akkor megpróbál gyógyulni
                        G[i][j] = (byte) (x+2);     //ha sikerül, akkor immunissá válik
                        infected--;
                    } else {
                        G[i][j]++;                  //különben telnek a beteg napok
                        if (G[i][j] == x+1) {       //nem gyógyult meg időben,
                            infected--;             //akkor elpusztult
                            dead++;
                        }
                    }
                }
                
            }
        }
        
        // várakozás véglegesítése
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                if (G[i][j] == -1) G[i][j] = 0; //ha meg lett fertőzve a körben, akkor előtör a betegség
            }
        }
        if (infected >= .80 * n*n) heal = false;
        else heal = true;
        // maximális arány figyelése, idő
        t++;
        if (infected > infectedMax) {
            infectedMax = infected;
            t_infectedMax = t;
        }
        
    }
    
    public void draw(Draw D) {
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                if (G[i][j] == -2) D.drawImage(TreeDisease.tree, i, j, .40); // egészséges
                else if (0 <= G[i][j] && G[i][j] <= x) D.drawImage(TreeDisease.disease, i, j, .40); // beteg
                else if (G[i][j] == x+1) D.drawImage(TreeDisease.dead, i, j, .40); // beteg
                else if (G[i][j] == x+2) D.drawImage(TreeDisease.healed, i, j, .40); // gyógyult
                else throw new RuntimeException("Invalid tree status detected at ("+i+","+j+").");
                //D.centeredText(G[i][j]+"", i, j);
            }
        }
    }
        
    public int getInfected() {
        return infected;
    }
    
    public boolean getHeal() {
        return heal;
    }
    
    public double run() {
        long t = 0;
        while (true) {
            step(); t++;
            //System.out.println(infected);
            if (infected == 0) {
                return (dead+0.0)/(n*n);
            }
            
            //if (infected >= .80 * n*n) return 1;
            //if (t == t_limit) return 0;
        }
    }
}

class GrdStats {
    static double count(int N, double L, int n, double p0, double z, int x, long t_limit, int r) {
        double count = 0;
        for (int i = 0; i < N; i++) {
            Garden garden = new Garden(L, n, p0, z, x, t_limit, r);
            count += garden.run();
            //System.out.print(".");
            //System.out.flush();
        }
        return (count+0.0)/N;
    }
    
    static double[] array(int N, int nmin, int nmax, double L, double p0, double z, int x, long t_limit, int r) {
        double[] array = new double[nmax-nmin+1];
        for (int n = nmin; n <= nmax; n++) {
            //System.out.print(n);
            array[n-nmin] = count(N, L, n, p0, z, x, t_limit, r);
            //System.out.println();
        }
        return array;
    }
}