package treedisease;

import java.awt.*;
import static java.awt.Color.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.text.NumberFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.concurrent.ExecutionException;
import javax.imageio.ImageIO;
import javax.swing.*;
import static javax.swing.JFileChooser.SAVE_DIALOG;
import javax.swing.event.*;

public class TreeDisease {
    static final BufferedImage tree, disease, healed, dead;
    static final int timerStep = 333;
    static long t = 0;
    static Timer T;
    private static JFrame frame;
    private static JPanel contentPane, leftPane, rightPane;
    static GardenPanel gardenPanel;
    private static JSlider LSlider, nSlider, p0Slider, xSlider, zSlider, tlSlider, rSlider, nminSlider, nmaxSlider;
    private static JButton pause, restart, graph, export, switchMode;
    private static JLabel statusLabel;
    static double L = 1; // L×L-es terület
    static int n = 10; // n×n-es kertrács
    static double p0 = 0.025; // kezdeti arány
    static double z = 0.01; 
    static int x = 7;
    static long t_limit = 3 * 365;
    static int r = 100;
    static int nmin = 1; static int nmax = 20;
    static Garden garden = new Garden(L, n, p0, z, x, t_limit, r);
    private static final Locale locale = Locale.GERMAN;
    static NumberFormat formatter, formatter3;
    static boolean graphMode = false;
    static double[][] graphArray = null;
    private static JFileChooser fileChooser;
    private static long timestamp;
    private static final int runsExport = 20;
    private static final int runsGraph = 100;

    
    static {
        tree = readImage("tree.png");
        disease = readImage("disease.png");
        healed = readImage("healed.png");
        dead = readImage("dead.png");
        // 1 tizedesre
        formatter = NumberFormat.getInstance(locale);
        formatter.setMaximumFractionDigits(1);
        formatter.setMinimumFractionDigits(1);
        // 3 tizedesre
        formatter3 = NumberFormat.getInstance(locale);
        formatter3.setMaximumFractionDigits(3);
        formatter3.setMinimumFractionDigits(3);
        
        fileChooser = new JFileChooser() {
            @Override
            public void approveSelection() { // létező fájlba mentés előtt megkérdezi, hogy felülírja-e
                File file = getSelectedFile();
                if (file.exists() && getDialogType() == SAVE_DIALOG) { // ha a fájl létezik és menteni próbálunk bele
                    // Megkérdezzük, hogy biztos-e
                    Object options[] = {"Igen", "Nem", "Mégse"};
                    ImageIcon icon = new ImageIcon(readImage("Mazsola.jpg").getScaledInstance(120, 80, Image.SCALE_SMOOTH));
                    int result = JOptionPane.showOptionDialog(// létrehozunk egy lehetőségeket felkínáló párbeszédablakot
                            this, // alapja a fileChooser
                            "A fájl már létezik. Felülírjuk?", // az üzenet szövege
                            "Fájl felülírása", // az ablak címe
                            JOptionPane.YES_NO_CANCEL_OPTION, // opciók: igen, nem, mégse
                            JOptionPane.WARNING_MESSAGE, // look and feel: figyelmeztetés
                            (icon == null ? null : icon),
                            options, // opciók tömbje
                            options[1] // default opció (ez van fókuszban)
                    ); 
                    switch(result) {
                        case JOptionPane.YES_OPTION: // igen
                            super.approveSelection();
                            return;
                        case JOptionPane.NO_OPTION: // nem
                            return;
                        case JOptionPane.CLOSED_OPTION: // ablak bezárva
                            return;
                        case JOptionPane.CANCEL_OPTION: // mégse
                            cancelSelection(); // a fileChoosert is bezárjuk
                            return;
                    }
                }
                super.approveSelection(); // ha a fájl még nem létezik, normál működés
            }
        };
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
    }
    
    private static void createAndShowGUI() {
        frame = new JFrame("Beteges kertecske");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setResizable(false);
        contentPane = new JPanel();
        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.LINE_AXIS));
        
        leftPane = new JPanel();
        leftPane.setLayout(new BoxLayout(leftPane, BoxLayout.PAGE_AXIS));
        gardenPanel = new GardenPanel();
        leftPane.add(gardenPanel);
        contentPane.add(leftPane);
        
        rightPane = new JPanel();
        rightPane.setLayout(new BoxLayout(rightPane, BoxLayout.PAGE_AXIS));
        rightPane.setPreferredSize(new Dimension(600,600));
        rightPane.add(Box.createRigidArea(new Dimension(0, 50)));
        initSlidersConstruct();
        initSliders();
        rightPane.add(Box.createRigidArea(new Dimension(0, 10)));
        initRatio();
        rightPane.add(Box.createRigidArea(new Dimension(0, 10)));
        initButtons();
        rightPane.add(Box.createRigidArea(new Dimension(0, 200)));
        contentPane.add(rightPane);
        
        frame.setContentPane(contentPane);
        frame.pack();
        frame.setVisible(true);
    }
    
    private static void initSlidersConstruct() {
        Font font = new Font("Sans", Font.PLAIN, 8);
        /*
        LSlider = new JSlider(JSlider.HORIZONTAL,1,3000,(int) (1000*L));
        LSlider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">L</span> = "+formatter3.format(L)+" km</html>"));
        Hashtable labelTableL = new Hashtable();
        for (int i = 1000; i <= 3000; i+=1000) {
            JLabel label = new JLabel(i/1000+" km");
            label.setFont(font);
            labelTableL.put(new Integer(i), label);
        }
        LSlider.setLabelTable(labelTableL);
        LSlider.setPaintLabels(true);
        LSlider.addChangeListener(sliderListener);
        */            
        nSlider = new JSlider(JSlider.HORIZONTAL,1,50,n);
        nSlider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">n</span> = "+n+"</html>"));
        Hashtable labelTableN = new Hashtable();
        for (int i = 5; i <= 50; i+=5) {
            JLabel label = new JLabel(i+"");
            label.setFont(font);
            labelTableN.put(new Integer(i), label);
        }
        nSlider.setLabelTable(labelTableN);
        nSlider.setPaintLabels(true);
        nSlider.addChangeListener(sliderListener);
        
        p0Slider = new JSlider(JSlider.HORIZONTAL,0,1000,(int) (1000*p0));
        p0Slider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">p</span><sub>0</sub> = "+formatter.format(100*p0)+"%</html>"));
        Hashtable labelTableP0 = new Hashtable();
        for (int i = 0; i <= 1000; i+=200) {
            JLabel label = new JLabel(i/10+"%");
            label.setFont(font);
            labelTableP0.put(new Integer(i), label);
        }
        p0Slider.setLabelTable(labelTableP0);
        p0Slider.setPaintLabels(true);
        p0Slider.addChangeListener(sliderListener);
        
        xSlider = new JSlider(JSlider.HORIZONTAL,1,14,x);
        xSlider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">x</span> = "+x+"</html>"));
        Hashtable labelTableX = new Hashtable();
        for (int i = 1; i <= 14; i+=1) {
            JLabel label = new JLabel(i+"");
            label.setFont(font);
            labelTableX.put(new Integer(i), label);
        }
        xSlider.setLabelTable(labelTableX);
        xSlider.setPaintLabels(true);
        xSlider.addChangeListener(sliderListener);
        
        zSlider = new JSlider(JSlider.HORIZONTAL,0,1000,(int) (1000*z));
        zSlider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">z</span> = "+(100*z)+"%</html>"));
        Hashtable labelTableTL = new Hashtable();
        for (int i = 0; i <= 1000; i+=200) {
            JLabel label = new JLabel(i/10+"%");
            label.setFont(font);
            labelTableTL.put(new Integer(i), label);
        }
        zSlider.setLabelTable(labelTableTL);
        zSlider.setPaintLabels(true);
        zSlider.addChangeListener(sliderListener);
        
        /*
        tlSlider = new JSlider(JSlider.HORIZONTAL,1,3*365,(int) t_limit);
        tlSlider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">t</span><sub>max</sub> = "+t_limit+"</html>"));
        Hashtable labelTableTL = new Hashtable();
        for (int i = 100; i <= 1000; i+=100) {
            JLabel label = new JLabel(i+"");
            label.setFont(font);
            labelTableTL.put(new Integer(i), label);
        }
        tlSlider.setLabelTable(labelTableTL);
        tlSlider.setPaintLabels(true);
        tlSlider.addChangeListener(sliderListener);
        */
        rSlider = new JSlider(JSlider.HORIZONTAL,1, 500, r);
        rSlider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">r/L</span> = "+r/1000.0+"</html>"));
        Hashtable labelTableR = new Hashtable();
        for (int i = 50; i <= 500; i+=50) {
            JLabel label = new JLabel(i/1000.0+"");
            label.setFont(font);
            labelTableR.put(new Integer(i), label);
        }
        rSlider.setLabelTable(labelTableR);
        rSlider.setPaintLabels(true);
        rSlider.addChangeListener(sliderListener);
        
        nminSlider = new JSlider(JSlider.HORIZONTAL,1,50,nmin);
        nminSlider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">n</span><sub>min</sub> = "+nmin+"</html>"));
        Hashtable labelTableNmin = new Hashtable();
        for (int i = 5; i <= 50; i+=5) {
            JLabel label = new JLabel(i+"");
            label.setFont(font);
            labelTableNmin.put(new Integer(i), label);
        }
        nminSlider.setLabelTable(labelTableNmin);
        nminSlider.setPaintLabels(true);
        nminSlider.addChangeListener(sliderListener);
        
        nmaxSlider = new JSlider(JSlider.HORIZONTAL,1,50,nmax);
        nmaxSlider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">n</span><sub>max</sub> = "+nmax+"</html>"));
        Hashtable labelTableNmax = new Hashtable();
        for (int i = 5; i <= 50; i+=5) {
            JLabel label = new JLabel(i+"");
            label.setFont(font);
            labelTableNmax.put(new Integer(i), label);
        }
        nmaxSlider.setLabelTable(labelTableNmax);
        nmaxSlider.setPaintLabels(true);
        nmaxSlider.addChangeListener(sliderListener);
        
    }
    
    private static void initSliders() {
        JPanel sliderPane1 = new JPanel();
        sliderPane1.setLayout(new BoxLayout(sliderPane1, BoxLayout.PAGE_AXIS));
        //sliderPane1.add(LSlider);
        sliderPane1.add(nSlider);
        sliderPane1.add(p0Slider);
        sliderPane1.add(xSlider);
        //sliderPane1.add(tlSlider);
        sliderPane1.add(zSlider);
        sliderPane1.add(rSlider);
        sliderPane1.add(nminSlider);
        sliderPane1.add(nmaxSlider);
        
        rightPane.add(sliderPane1);
    }
    
    private static void initRatio() {
        JPanel statusPanel = new JPanel();
        String s = "<html>A(z) "+t+". napon a beteg fák aránya "+formatter.format(100*garden.getInfected()/(n*n+0.0))+"%, "
                        + "maximális fertőzöttség a(z) "+garden.t_infectedMax+". napon, aránya "
                        + formatter.format(100*garden.infectedMax/(n*n+0.0))+"%.</html>";
        statusLabel = new JLabel(s);
        statusPanel.add(statusLabel);
        rightPane.add(statusPanel);
    }
    
    private static void initButtons() {
        JPanel buttonsPane = new JPanel();
        buttonsPane.setLayout(new BoxLayout(buttonsPane, BoxLayout.LINE_AXIS));
        
        pause = new JButton("START");
        pause.setMnemonic(KeyEvent.VK_S);
        pause.addActionListener(buttonListener);
        
        restart = new JButton("Új kert");
        restart.setMnemonic(KeyEvent.VK_R);
        restart.addActionListener(buttonListener);
        
        graph = new JButton("Grafikon generálása");
        graph.setMnemonic(KeyEvent.VK_G);
        graph.setEnabled(false);
        graph.addActionListener(buttonListener);
        
        export = new JButton("Exportálás 3D");
        export.setMnemonic(KeyEvent.VK_X);
        export.addActionListener(buttonListener);
        
        switchMode = new JButton("Váltás grafikonra");
        switchMode.setMnemonic(KeyEvent.VK_V);
        switchMode.addActionListener(buttonListener);
        
        buttonsPane.add(pause);
        buttonsPane.add(Box.createRigidArea(new Dimension(5,0)));
        buttonsPane.add(restart);
        buttonsPane.add(Box.createRigidArea(new Dimension(5,0)));
        buttonsPane.add(graph);
        buttonsPane.add(Box.createRigidArea(new Dimension(5,0)));
        buttonsPane.add(export);
        buttonsPane.add(Box.createRigidArea(new Dimension(5,0)));
        buttonsPane.add(switchMode);
        rightPane.add(buttonsPane);
    }
    
    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                createAndShowGUI();
                T = new Timer(timerStep, updateTimerAction);
                T.setInitialDelay(timerStep);
                T.setCoalesce(false);
            }
        });
    }
    
    static BufferedImage readImage(String s) {
        BufferedImage img = null;
        try {
            img = ImageIO.read(TreeDisease.class.getResourceAsStream("/images/"+s));
        } catch (IOException | IllegalArgumentException e) {
            System.err.println("Hiba a kép olvasásánál.");
        }
        return img;
    }
    
    private static final ActionListener updateTimerAction = new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            t++;
            garden.step();
            draw();
            check();
        }

        private void check() {
            if (garden.getInfected() == 0) {
                draw();
                T.stop();
                pause.setText("START");
                t = 0;
            }
        }
    };
    
    private static final ActionListener buttonListener = new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            Object source = e.getSource();
            if (source.equals(pause)) {
                if (!T.isRunning()) {
                    pause.setText("PAUSE");
                    T.restart();
                } else {
                    T.stop();
                    pause.setText("START");
                }
            }
            if (source.equals(restart)) {
                newGarden();
            }
            if (source.equals(graph)) {
                if (nmin >= nmax) {
                    JOptionPane.showMessageDialog(frame,
                        "<html><span style=\"font-family: serif; font-style: italic;\">n</span><sub>min</sub>-nek kisebbnek kell lennie"
                                + "<span style=\"font-family: serif; font-style: italic;\">n</span><sub>max</sub>-nál.</html>",
                        "Hiba",
                        JOptionPane.ERROR_MESSAGE);
                    return;
                }
                graph.setEnabled(false);
                switchMode.setEnabled(false);
                statusLabel.setText("Grafikon generálása folyamatban…");
                
                SwingWorker graphWorker = new SwingWorker<double[][], Integer>() {
                    @Override
                    protected double[][] doInBackground() throws Exception {
                        timestamp = System.currentTimeMillis();
                        int numOfDataSets = 1;
                        double array[][] = new double[numOfDataSets][];
                        for (int i = 0; i < numOfDataSets; i++) {
                            array[i] = new double[nmax-nmin+1];
                            for (int n = nmin; n <= nmax; n++) {
                                array[i][n-nmin] = GrdStats.count(runsGraph, L, n, p0, z, x, t_limit, r);
                                publish(n);
                            }
                        }
                        return array;
                    }

                    @Override
                    protected void process(java.util.List<Integer> list) {
                        Integer I = list.get(list.size()-1);
                        statusLabel.setText("<html>Grafikon generálása folyamatban… "
                                + "(<span style=\"font-family: serif; font-style: italic;\">n</span> = "+I+")</html>");
                    }
                    
                    @Override
                    protected void done() {
                        try {
                            graphArray = get();
                            draw();
                        } catch (InterruptedException | ExecutionException ex) {}
                        graph.setEnabled(true);
                        switchMode.setEnabled(true);
                        statusLabel.setText(runsGraph+" futásból átlagolva a grafikon generálása "
                                +formatter.format((System.currentTimeMillis()-timestamp)/1000.0)+" s-ig tartott");
                    }
                };
                graphWorker.execute();
                //StdDraw.setPenRadius(.01);
                //for (int i = 0; i < 3; i++) StdStats.plotLines(GrdStats.array(150, 2, 20, L, p0, x, t_limit));
            }
            if (source.equals(export)) {
                if (nmin > nmax) {
                    JOptionPane.showMessageDialog(frame,
                        "<html><span style=\"font-family: serif; font-style: italic;\">n</span><sub>min</sub> nem lehet nagyobb "
                                + "<span style=\"font-family: serif; font-style: italic;\">n</span><sub>max</sub>-nál.</html>",
                        "Hiba",
                        JOptionPane.ERROR_MESSAGE);
                    return;
                }
                export.setEnabled(false);
                
                SwingWorker exportWorker = new SwingWorker<Void, Void>() {
                    @Override
                    protected Void doInBackground() {            
                        int returnVal = fileChooser.showSaveDialog(frame);

                        if (returnVal == JFileChooser.APPROVE_OPTION) { // ha a fájl kiválasztása megtörtént
                            JOptionPane.showMessageDialog(frame, "Az exportálás elkezdődött. Beállításoktól függően akár sok ideig is eltarthat.",
                                        "Exportálás elkezdődött", JOptionPane.INFORMATION_MESSAGE);
                            
                            File file = fileChooser.getSelectedFile();
                            
                            try (PrintWriter writer = new PrintWriter(file, "UTF-8")) {
                                double zmin = 0.0, zmax = 0.5, zdiff = 0.02;
                                writer.println("n: ["+nmin+"; "+nmax+"], z: ["+zmin+"; "+zmax+"] increment = "+zdiff+", p0 = "+p0+", x = "+x+", r/L = "+ r/1000.0 +". Pontosság: " + runsExport + " futtatás.");
                                String headline = "n\\z\t|";
                                String separator = "____|";
                                for(double z1 = zmin; z1 < zmax+0.0001; z1 += zdiff) {
                                    headline += "\t"+formatter3.format(z1);
                                    separator += "________";
                                }
                                writer.println(headline);
                                writer.println(separator);
                                for (int n = nmin; n <= nmax; n++) {
                                    String s = n + "\t|";
                                    for(double z1 = zmin; z1 < zmax+0.0001; z1 += zdiff){
                                        double[] array = GrdStats.array(runsExport, nmin, nmax, L, p0, z1, x, t_limit, r);
                                        s += "\t"+formatter3.format(array[n-nmin]);
                                        System.out.println("\tz = "+z1+" done");
                                    }
                                    writer.println(s);
                                    System.out.println("n = "+n+" done");
                                }
                                writer.close();
                                JOptionPane.showMessageDialog(frame, "Az exportálás sikeresen befejeződött.",
                                        "Exportálás vége", JOptionPane.INFORMATION_MESSAGE);
                            } catch (FileNotFoundException ex) {
                                System.err.println("Hiba a fájl írásánál");
                            } catch (UnsupportedEncodingException ex) {
                                System.err.println("Hiba a fájl írásánál: ismeretlen karakterkódolás");
                            }
                        }
                        return null;
                    }

                    @Override
                    protected void done() {
                        export.setEnabled(true);
                    }
                };
                exportWorker.execute();
            }
            if (source.equals(switchMode)) {
                if (graphMode) { // graph >> animate
                    graphMode = false;
                    pause.setEnabled(true);
                    restart.setEnabled(true);
                    graph.setEnabled(false);
                    statusLabel.setText(" ");
                    switchMode.setText("Váltás grafikonra");
                } else { // animate >> graph
                    graphMode = true;
                    T.stop();
                    pause.setText("START");
                    pause.setEnabled(false);
                    restart.setEnabled(false);
                    graph.setEnabled(true);
                    statusLabel.setText(" ");
                    switchMode.setText("Váltás animációra");
                }
                draw();
            }
        }
    };
    
    private static final ChangeListener sliderListener = new ChangeListener() {
        @Override
        public void stateChanged(ChangeEvent e) {
            JSlider source = (JSlider) e.getSource();
            if (!source.getValueIsAdjusting()) {/*
                if (source.equals(LSlider)) {
                    L = LSlider.getValue() / 1000.0;
                    LSlider.setBorder(BorderFactory.createTitledBorder(
                "<html><span style=\"font-family: serif; font-style: italic;\">L</span> = "+formatter3.format(L)+" km</html>"));
                }*/
                if (source.equals(nSlider)) {
                    n = nSlider.getValue();
                    nSlider.setBorder(BorderFactory.createTitledBorder(
                       "<html><span style=\"font-family: serif; font-style: italic;\">n</span> = "+n+"</html>"));
                }
                if (source.equals(p0Slider)) {
                    p0 = p0Slider.getValue()/1000.0;
                    p0Slider.setBorder(BorderFactory.createTitledBorder(
                        "<html><span style=\"font-family: serif; font-style: italic;\">p</span><sub>0</sub> = "+formatter.format(100*p0)+"%</html>"));
                }
                if (source.equals(xSlider)) {
                    x = xSlider.getValue();
                    xSlider.setBorder(BorderFactory.createTitledBorder(
                        "<html><span style=\"font-family: serif; font-style: italic;\">x</span> = "+x+"</html>"));
                }/*
                if (source.equals(tlSlider)) {
                    t_limit = tlSlider.getValue();
                    tlSlider.setBorder(BorderFactory.createTitledBorder(
                        "<html><span style=\"font-family: serif; font-style: italic;\">t</span><sub>max</sub> = "+t_limit+"</html>"));
                }*/
                if (source.equals(zSlider)) {
                    z = zSlider.getValue()/1000.0;
                    zSlider.setBorder(BorderFactory.createTitledBorder(
                        "<html><span style=\"font-family: serif; font-style: italic;\">z</span> = "+formatter.format(100*z)+"%</html>"));
                }
                if (source.equals(rSlider)) {
                    r = rSlider.getValue();
                    rSlider.setBorder(BorderFactory.createTitledBorder(
                        "<html><span style=\"font-family: serif; font-style: italic;\">r/L</span> = "+r/1000.0+"</html>"));
                }
                if (source.equals(nminSlider)) {
                    nmin = nminSlider.getValue();
                    nminSlider.setBorder(BorderFactory.createTitledBorder(
                       "<html><span style=\"font-family: serif; font-style: italic;\">n</span><sub>min</sub> = "+nmin+"</html>"));
                }
                if (source.equals(nmaxSlider)) {
                    nmax = nmaxSlider.getValue();
                    nmaxSlider.setBorder(BorderFactory.createTitledBorder(
                       "<html><span style=\"font-family: serif; font-style: italic;\">n</span><sub>max</sub> = "+nmax+"</html>"));
                }
            }
            newGarden();
        }
    };
    
    private static void draw() {
        if (graphMode) {
            gardenPanel.paintImmediately(0, 0, gardenPanel.getWidth(), gardenPanel.getHeight());
        } else {
            gardenPanel.paintImmediately(0, 0, gardenPanel.getWidth(), gardenPanel.getHeight());
            if (garden.getHeal()) 
                statusLabel.setText("<html>A(z) "+t+". napon a beteg fák aránya "+formatter.format(100*garden.getInfected()/(n*n+0.0))+"%, "
                        + "maximális fertőzöttség a(z) "+garden.t_infectedMax+". napon, aránya "
                        + formatter.format(100*garden.infectedMax/(n*n+0.0))+"%.</html>");
            else
                statusLabel.setText("<html>A(z) "+t+". napon a beteg fák aránya "
                        + "<span style=\"color: red;\">"+formatter.format(100*garden.getInfected()/(n*n+0.0))+"%</span>, "
                        + "maximális fertőzöttség a(z) "+garden.t_infectedMax+". napon, aránya "
                        + formatter.format(100*garden.infectedMax/(n*n+0.0))+"%.</html>");
        }
    }
    
    private static void newGarden() {
        T.stop();
        pause.setText("START");
        t = 0;
        garden = new Garden(L, n, p0, z, x, t_limit, r);
        draw();
    }
    
    private static class GardenPanel extends JPanel {    
        public GardenPanel() {
            setBorder(BorderFactory.createLineBorder(BLACK));
            setBackground(WHITE);
        }

        @Override
        public Dimension getPreferredSize() {
            return new Dimension(600,600);
        }

        @Override
        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            Draw D = new Draw((Graphics2D) g,600,600);
            if (graphMode) { // grafikon
                D.setXscale(0, nmax-nmin); // ezt a Stats.plotLines() meghívása felülírhatja
                D.setYScale(0, 1);
                D.setPenRadius(.003);
                if (graphArray != null) {
                    for (double[] array : graphArray) Stats.plotLinesHelp(array, D);
                }
                D.setPenRadius();
                // x-tengely
                D.line(0, 0, nmax-nmin, 0);
                D.setFont(new Font("Sans", Font.PLAIN, 10));
                for (int i = nmin; i <= nmax; i++) {
                    D.line(i-nmin, 0, i-nmin, -.005);
                    D.centeredText(i+"", i-nmin, -.025);
                }
                // y-tengely
                D.line(0, 0, 0, 1);
                for (int i = 0; i <= 10; i++) {
                    double w = (nmax-nmin)/75.0;
                    D.line(0, i/10.0, -w, i/10.0);
                }
                for (int i = 0; i <= 20; i++) {
                    double w = (nmax-nmin)/200.0;
                    D.line(0, i/20.0, -w, i/20.0);
                }
                
                
                D.setPenColor(Color.RED);
                if (graphArray != null) {
                    for (double[] array : graphArray) {
                        g.drawString(formatter.format(array[0]*100.0)+"%", 3, -30+ 600-(int)((600-50)*array[0]));
                        g.drawString(formatter.format(array[nmax-nmin]*100.0)+"%", 3, -30+ 600-(int)((600-50)*array[nmax-nmin]));
                    }
                }
                D.setPenColor();
                
                g.drawString("100%", 3, 25);
                g.drawString("0%", 3, 600-20);
                // címkék
                D.setFont(new Font("Serif", Font.ITALIC, 13));
                g.drawString("n", 600-15, 600-20);
                g.drawString("q", 20, 10);
            } else { // kert
                D.setXscale(-0.5, garden.n-0.5);
                D.setYScale(-0.5, garden.n-0.5);
                //g.drawString(t+". nap", 15, 600-15);
                //g.drawString(formatter.format(100*garden.getInfected()/(n*n+0.0))+"% beteg", 600-85, 600-15);
                garden.draw(D);
            }
        }
    }
}